# Generated by Django 2.2.5 on 2019-09-27 05:15

from django.db import migrations
import picklefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('django_kck', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataproduct',
            name='value',
            field=picklefield.fields.PickledObjectField(editable=False),
        ),
        migrations.AlterField(
            model_name='refreshqueue',
            name='hints',
            field=picklefield.fields.PickledObjectField(blank=True, editable=False, null=True),
        ),
    ]
